@@ -2,6 +2,7 @@
 #include <QtGui/QApplication>
 #include "qmlapplicationviewer.h"
 #include "windowhelper.h"
+#include "filehandler.h"
 
 #ifdef HAVE_GLWIDGET
 
@@ -22,7 +23,8 @@ int main(int argc, char *argv[])
 
     WindowHelper *windowHelper = new WindowHelper();
     viewer.rootContext()->setContextProperty("windowHelper", windowHelper);
-    //viewer.rootContext()->setContextProperty("fileHandler", fileHandler);
+    FileHandler *fileHandler = new FileHandler();
+    viewer.rootContext()->setContextProperty("fileHandler", fileHandler);
 
     viewer.setAttribute(Qt::WA_NoSystemBackground);
     //viewer.setAttribute(Qt::WA_OpaquePaintEvent);
