@@ -1,7 +1,7 @@
 /*
     Copyright 2011 - Tommi Laukkanen (www.substanceofcode.com)
 
-    This file is part of NewsFlow.
+    This file is modified from Tommi Laukkanen's NewsFlow.
 
     NewsFlow is free software: you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -136,7 +136,8 @@ function doWebRequest(method, url, params, callback, show) {
     doc.onreadystatechange = function() {
         if (doc.readyState == XMLHttpRequest.HEADERS_RECEIVED) {
             var status = doc.status;
-            if(status!=200) {
+            // Ignore 500 error
+            if(status!=200&&status!=500) {
                 showError("API returned " + status + " " + doc.statusText, show);
             }
         } else if (doc.readyState == XMLHttpRequest.DONE) {
@@ -180,7 +181,8 @@ function doWebRequestIdx(method, url, params, callback, idx, show) {
     doc.onreadystatechange = function() {
         if (doc.readyState == XMLHttpRequest.HEADERS_RECEIVED) {
             var status = doc.status;
-            if(status!=200) {
+            // Ignore 500 error
+            if(status!=200&&status!=500) {
                 showError("Google API returned " + status + " " + doc.statusText, show);
             }
         } else if (doc.readyState == XMLHttpRequest.DONE) {
@@ -249,14 +251,17 @@ function parseToken(data) {
         waiting.state = "portfolio";
     } else {
         //addError("Couldn't parse SID");
-        addError("Login failed.\nCheck username/password");
+        addError("Login failed.\nCheck username/password or \nnetwork connection");
         waiting.state = "hidden";
     }
 }
 
 function login(email, password) {
+    error.reason = "";
+    error.state  = "hidden";
+
     try {
-        waiting.state = "login";
+        waiting.state = "shown";
         var url = "https://www.google.com/accounts/ClientLogin?Email=" + encodeURIComponent(email) + "&Passwd=" + encodeURIComponent(password) + "&service=finance" + "&source=" + encodeURIComponent("stockona");
         doWebRequest("POST", url, 1, parseToken, 0);
     }catch(err) {
@@ -295,7 +300,7 @@ function loadOnePosition(posFeedLink) {
 }
 
 function loadOneQuote(symObj, idx) {
-    itemsURL = "http://www.google.com/finance/info?client=ig&infotype=infoquoteall&q="+symObj.symName;
+    itemsURL = "http://www.google.com/finance/info?client=ig&infotype=infoquoteall&q=" + symObj.symExg + ":" + symObj.symName;
     //console.log(itemsURL);
     loadQuote(symObj, idx);
 }
@@ -323,6 +328,15 @@ function loadPortfolio() {
     }
 }
 
+function loadPortfolioNum(pfoIdx) {
+    try {
+        waiting.state = "shown";
+        doWebRequestIdx("GET", itemsURL, 0, parsePortfolioNum, pfoIdx, 1);
+    } catch(err) {
+        showError("Error while loading portfolio num: " + err, 1);
+    }
+}
+
 function loadPosition() {
     try {
         waiting.state = "shown";
@@ -333,18 +347,17 @@ function loadPosition() {
 }
 
 function loadQuote(symObj, idx) {
-    var rtnYTDTrunc = sprintf("%.2f", 100*symObj.symRtnYTD);
-
     if (idx >= pModel.count) {
         pModel.append({
                      "idx":            idx,
                      "id":             symObj.symId,
-                     "gainPercent":    "0",
-                     "gainPercentFull":symObj.symGainPercent,
-                     "rtnYTD":         rtnYTDTrunc,
+                     "rtnYTD":         symObj.symRtnYTD,
                      "share":          symObj.symShare,
                      "shareCost":      symObj.shareCost,
+                     "shareDayGain":   symObj.shareDayGain,
+                     "shareGain":      symObj.shareGain,
                      "shareValue":     symObj.shareValue,
+                     "shareGainPercent": symObj.shareGainPercent,
 
                      "exchange":       "-",
                      "fullName":       "-",
@@ -377,11 +390,12 @@ function loadQuote(symObj, idx) {
     else {
         pModel.set(idx, {
             "share":          symObj.symShare,
-            "gainPercentFull":symObj.symGainPercent,
-            "rtnYTD":         rtnYTDTrunc, //symObj.symRtnYTD
+            "shareGain":      symObj.shareGain,
             "shareCost":      symObj.shareCost,
-            "shareValue":     symObj.shareValue
-
+            "shareDayGain":   symObj.shareDayGain,
+            "shareValue":     symObj.shareValue,
+            "rtnYTD":         symObj.symRtnYTD,
+            "shareGainPercent": symObj.shareGainPercent
         });
     }
 
@@ -438,32 +452,40 @@ function parsePortfolio(data) {
             var pfoRtnYTD      = data.childNodes[11+i].childNodes[8].attributes[9].nodeValue;
 
             // Push to pfoModel
-//            if (i<=model.count) {
-//                model.set(i, {
-//                             "idx": i,
-//                             "name": pfoName,
-//                             "etag": pfoEtag,
-//                             "id": pfoId,
-//                             "feedLink": pfoFeedLink,
-//                             "currency": currency,
-//                             "gainPercent": pfoGainPercent,
-//                             "rtnYTD": pfoRtnYTD
-
-//                });
-//            }
-//            else {
-                model.append({
-                             "idx": i,
+            if (i<model.count) {
+                model.set(i, {
+                             //"idx": i,
+                             //"etag": pfoEtag,
                              "name": pfoName,
-                             "etag": pfoEtag,
-                             "id": pfoId,
+                             //"id": pfoId,
                              "feedLink": pfoFeedLink,
-                             "currency": currency,
-                             "gainPercent": pfoGainPercent,
-                             "rtnYTD": pfoRtnYTD
-
+                             //"currency": currency,
+                             //"gainPercent": pfoGainPercent,
+                             //"rtnYTD": pfoRtnYTD,
+                             "excerpt": "-",
+                             "num": 0
                 });
-//            }
+            }
+            else {
+                model.append({
+                             //"idx": i,
+                             //"etag": pfoEtag,
+                             "local": 0,
+                             "name": pfoName,
+                             //"id": pfoId,
+                             "feedLink": pfoFeedLink,
+                             //"currency": currency,
+                             //"gainPercent": pfoGainPercent,
+                             //"rtnYTD": pfoRtnYTD,
+                             "excerpt": "-",
+                             "num": 0
+                });
+            }
+
+            // Retrieve number of quotes for each portfolio... this is kinda redundant but do with it for now.
+            var idx = i + 1;
+            itemsURL = pfoId + "/positions";
+            loadPortfolioNum(i);
 
             /*
             console.log("<Portfolio.entry>");
@@ -474,19 +496,54 @@ function parsePortfolio(data) {
             console.log("currency="+currency);
             console.log("pfo.gainPercent="+pfoGainPercent);
             console.log("pfo.rtnYTD="+pfoRtnYTD);
-            */
+            //*/
         }
-
     //}catch(err) {
     //    addError("Error: " + err);
     //}
+        // Push local portfolio
+        var fileOk = fileHandler.loadPfo();
+
+        if (fileOk==0){
+            model.append({
+                         "local": 1,
+                         "name": fileHandler.localPfoName,
+                         "feedLink": "",
+                         "excerpt": "Locally stored portfolio",
+                         "num": 0
+            });
+        }
+
     waiting.state = "hidden";
 }
 
+function parsePortfolioNum(data, pfoIdx) {
+    var posLength = data.childNodes[7].firstChild.nodeValue;
+    var symName = new String("");
+
+    for (var i=0; i< ((posLength>3) ? 3 : posLength) ; i++) {
+        symName += data.childNodes[10+i].childNodes[7].attributes[2].nodeValue + ", ";
+    }
+
+    var lastComma = symName.lastIndexOf(",");
+    symName = symName.substring(0, lastComma);
+    symName +=  " ...";
+
+    // Push to pfoModel
+    //console.log("idx="+pfoIdx+" num="+posLength);
+    if (pfoIdx<model.count) {       
+        model.set(pfoIdx, {
+                  "num": posLength,
+                  "excerpt": symName
+        });
+    }
+}
+
 function parsePosition(data) {
     //try {
         // 2) XML DOM
         var posLength = data.childNodes[7].firstChild.nodeValue;
+
         //console.log("pos.length="+posLength);
         var quoteList = "";
 
@@ -574,29 +631,31 @@ function parsePosition(data) {
             }
 
             var symObj = {
-                "symId"          : data.childNodes[10+i].childNodes[0].firstChild.nodeValue,
-                "symGainPercent" : data.childNodes[10+i].childNodes[6].attributes[0].nodeValue,
-                "symRtnYTD"      : data.childNodes[10+i].childNodes[6].attributes[8].nodeValue,
+                //"symId"          : data.childNodes[10+i].childNodes[0].firstChild.nodeValue,
+                "shareGainPercent" : sprintf("%.2f", 100*data.childNodes[10+i].childNodes[6].attributes[0].nodeValue), // Same as returnOverall
+                "symRtnYTD"      : sprintf("%.2f", 100*data.childNodes[10+i].childNodes[6].attributes[8].nodeValue),
                 "symShare"       : data.childNodes[10+i].childNodes[6].attributes[9].nodeValue,
+                "symExg"         : data.childNodes[10+i].childNodes[7].attributes[0].nodeValue,
                 "symName"        : data.childNodes[10+i].childNodes[7].attributes[2].nodeValue,
 
-                "shareCost"  : "0",
-                "shareValue" : "0",
-                //"shareGain"  : "0"
+                "shareCost"     : "0",
+                "shareValue"    : "0",
+                "shareDayGain"  : "0",
+                "shareGain"     : "0"
 
                 // Don't parse for now...
                 //"symEtag"        : data.childNodes[10+i].attributes[0].nodeValue,
                 //"symFeedLink"    : data.childNodes[10+i].childNodes[5].attributes[0].nodeValue,   // transaction link
 
-                //"symExg"         : data.childNodes[10+i].childNodes[7].attributes[0].nodeValue,
                 //"symFullName"    : data.childNodes[10+i].childNodes[7].attributes[1].nodeValue,
             };
 
             // costBasis, gain, mktValue. Only those who have TX records are available
             if (data.childNodes[10+i].childNodes[6].childNodes.length==4) {
-                symObj.shareCost   = data.childNodes[10+i].childNodes[6].childNodes[0].firstChild.attributes[0].nodeValue;
-                //symObj.shareGain   = data.childNodes[10+i].childNodes[6].childNodes[2].firstChild.attributes[0].nodeValue;
-                symObj.shareValue  = data.childNodes[10+i].childNodes[6].childNodes[3].firstChild.attributes[0].nodeValue;
+                symObj.shareCost    = data.childNodes[10+i].childNodes[6].childNodes[0].firstChild.attributes[0].nodeValue;
+                symObj.shareDayGain = data.childNodes[10+i].childNodes[6].childNodes[1].firstChild.attributes[0].nodeValue;
+                symObj.shareGain   = data.childNodes[10+i].childNodes[6].childNodes[2].firstChild.attributes[0].nodeValue;
+                symObj.shareValue   = data.childNodes[10+i].childNodes[6].childNodes[3].firstChild.attributes[0].nodeValue;
                 //symObj.symCostUnit = data.childNodes[10+i].childNodes[6].childNodes[0].firstChild.attributes[1].nodeValue;
             }
 
@@ -605,14 +664,14 @@ function parsePosition(data) {
             // JSON Parsing, parsed data stored in global jsonObj.
             // Push needs to be done in parseJSON to keep the program synchronous.
             loadOneQuote(symObj, i);
-            quoteList = quoteList + "+" + symObj.symName;
+            quoteList = quoteList + "+" + convertExchangeForYahoo(symObj.symName, symObj.symExg);
 
             /*
             console.log("<position.entry>");
             //console.log("pos.etag="+symObj.symEtag);
             console.log("pos.id="+symObj.symId);
             console.log("pos.feedLink="+symObj.symFeedLink);
-            console.log("pos.gainPercent="+symObj.symGainPercent);
+            console.log("pos.gainPercent="+symObj.shareGainPercent);
             console.log("pos.rtnYTD="+symObj.symRtnYTD);
             console.log("pos.share="+symObj.symShare);
             console.log("pos.exchange="+symObj.symExg);
@@ -686,12 +745,12 @@ function parseJSON(data, idx) {
             quoteChgColor = "green";
         }
 
+        /*
         // Compute share value and cost for now...
         var gainPtg = pModel.get(idx).gainPercentFull;
         // truncate precision (or error to be exact)
         var gainPtgTrunc = sprintf("%.2f", gainPtg*100);
 
-        /*
         var share = pModel.get(idx).share;
         var shareValue = sprintf("%.4f", share*jsonObj[0].l);
         console.log(jsonObj[0].t+" "+share+" "+jsonObj[0].l);
@@ -717,14 +776,12 @@ function parseJSON(data, idx) {
 
     // Check if entry already exists, if not append, if yes set.
     // Use set method to avoid flickering when updating content by timer
-
-    if (idx<=pModel.count) {
+    //if (idx<pModel.count) {
 
         pModel.set(idx, {
                      "exchange":       (jsonObj[0].e   =="") ? "-" : jsonObj[0].e   ,    //symObj.symExg,
                      "fullName":       (jsonObj[0].name=="") ? "-" : jsonObj[0].name, //symObj.symFullName,
                      "name":           (jsonObj[0].t   =="") ? "-" : jsonObj[0].t   ,    //symObj.symName,
-                     "gainPercent":    (gainPtgTrunc   =="") ? "-" : gainPtgTrunc   ,
                      "quotePrice":     (jsonObj[0].l   =="") ? "-" : jsonObj[0].l   ,
                      "quoteChgColor":  (quoteChgColor  =="") ? "-" : quoteChgColor  ,
                      "quoteChg":       (jsonObj[0].c   =="") ? "-" : jsonObj[0].c   ,
@@ -741,36 +798,8 @@ function parseJSON(data, idx) {
                      "quotePe":        (jsonObj[0].pe  =="") ? "-" : jsonObj[0].pe  ,
                      "quoteType":      (jsonObj[0].type=="") ? "-" : jsonObj[0].type
         });
-    }
-    /*
-    else {
-        //pModel.insert(idx, {
-        pModel.append({
-                     "exchange":       jsonObj[0].e,    //symObj.symExg,
-                     "fullName":       jsonObj[0].name, //symObj.symFullName,
-                     "name":           jsonObj[0].t,    //symObj.symName,
+    //}
 
-                     "quotePrice":     jsonObj[0].l,
-
-                     "quoteChgColor":  quoteChgColor,
-                     "quoteChg":       jsonObj[0].c,
-                     "quoteChgPtg":    jsonObj[0].cp,
-                     "quoteVol":       jsonObj[0].vo,
-                     "quoteAvgVol":    jsonObj[0].avvo,
-                     "quoteMktCap":    jsonObj[0].mp,
-
-                     "quoteDayHi":     jsonObj[0].hi,
-                     "quoteDayLo":     jsonObj[0].lo,
-                     "quote52wHi":     jsonObj[0].hi52,
-                     "quote52wLo":     jsonObj[0].lo52,
-
-                     "quoteEps":       jsonObj[0].eps,
-                     "quoteBeta":      jsonObj[0].beta,
-                     "quotePe":        jsonObj[0].pe,
-                     "quoteType":      jsonObj[0].type
-        });
-    }
-    */
     waiting.state = "hidden";
 }
 
@@ -778,6 +807,24 @@ function cleanModel(model) {
     model.clear();
 }
 
+// Yahoo return 0 for ask/bid for non-North American exchange
+function convertExchangeForYahoo (name, exg) {
+    //console.log(name + exg);
+    if (exg=="NYSE" || exg=="NASDAQ") { return name; }
+    else if (exg=="OTC")              { return (name + "." + "OB"); }
+    else if (exg=="AMEX")             { return (name + "." + "OB"); }
+    else if (exg=="TSXV")             { return (name + "." + "V");  }
+    else if (exg=="TSX")              { return (name + "." + "TO"); }
+    else if (exg=="LON")              { return (name + "." + "L"); }
+    // Asian
+    else if (exg=="TPE")              { return (name + "." + "TW"); }
+    else if (exg=="HKG")              { return (name + "." + "HK"); }
+    else if (exg=="SHA")              { return (name + "." + "SS"); }
+    else if (exg=="SHE")              { return (name + "." + "SZ"); }
+    else if (exg=="SEO")              { return (name + "." + "KS"); }
+    else                              { return "NULL"; }
+}
+
 function getNodeValue(node, name) {
     var nodeValue = "";
     for(var i=0; i<node.childNodes.length; i++) {
